/*****************************************************************************
 *       FreeBar.c
 *       Windows Free Memory Program -
 *              This program will display a 1/2 icon bar in the middle
 *              of the icon displaying how much free memory
 *              is available in percentage.
 *****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define IDM_ABOUT   1

long FAR PASCAL WndProc(HWND, WORD, WORD, LONG);

HBRUSH          hBarBrush, hBackgroundBrush;
static  char    szAppName[] = "Free Memory Bar";
static  int     xCXICON,yCYICON,xCXBORDER,yCYBORDER;
static  WORD    wTotalMem;
static  float   fMemRatio;

int PASCAL WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
HANDLE  hInstance, hPrevInstance;
LPSTR   lpszCmdLine;
int     nCmdShow;
{
	HDC             hDC;
	TEXTMETRIC      tm;
	HWND            hWnd;
	WNDCLASS        wndclass;
	MSG             msg;
	HMENU           hMenu;

	if (!hPrevInstance) {
		wndclass.style          = CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc    = WndProc;
		wndclass.cbClsExtra     = 0;
		wndclass.cbWndExtra     = 0;
		wndclass.hInstance      = hInstance;
		wndclass.hIcon          = NULL;
		wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
		wndclass.hbrBackground  = NULL;
		wndclass.lpszMenuName   = NULL;
		wndclass.lpszClassName  = szAppName;

		if (!RegisterClass(&wndclass)) return FALSE;
	};
	hWnd = CreateWindow(szAppName, "FreeBar",
						WS_OVERLAPPEDWINDOW,
						CW_USEDEFAULT, 0,
						CW_USEDEFAULT, 0,
						NULL, NULL, hInstance, NULL);

	if (!SetTimer(hWnd, 1, 1000, NULL)) {
		MessageBox(hWnd, "Too many clocks or timers!", szAppName, MB_ICONHAND | MB_OK);
		return FALSE;
	};			
	hMenu = GetSystemMenu(hWnd,FALSE);
	AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
	AppendMenu(hMenu, MF_STRING, IDM_ABOUT, "About...");

	ShowWindow(hWnd, SW_SHOWMINNOACTIVE);
	UpdateWindow(hWnd);
	while(GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	};
	return msg.wParam;
}

long FAR PASCAL WndProc(HWND hWnd, WORD iMessage, WORD wParam, LONG lParam)
{
static  WORD    wFreeMem, wPrevMem = 0;
char            cBuffer[40];
HDC             hDC;
PAINTSTRUCT     ps;
short           i ;
switch(iMessage) {
	case    WM_CREATE:
		xCXICON = GetSystemMetrics(SM_CXICON);
		yCYICON = GetSystemMetrics(SM_CYICON);
		xCXBORDER = GetSystemMetrics(SM_CXBORDER);
		yCYBORDER = GetSystemMetrics(SM_CYBORDER);
		hBarBrush = CreateSolidBrush(GetSysColor(COLOR_APPWORKSPACE));
		hBackgroundBrush = CreateSolidBrush(GetSysColor(COLOR_BACKGROUND));
		SetClassWord(hWnd,GCW_HBRBACKGROUND,hBackgroundBrush);
		wTotalMem = (WORD) (GetFreeSpace (0) / 1024);
		break;

	case    WM_SYSCOMMAND:
		switch(wParam) {
			case    IDM_ABOUT:
				MessageBox(hWnd, "Free Memory Bar Program  Version 1.01\n"
						 "By Donald J. Byrd  CIS ID: 76357,1130\n"
                                                 "Win 3.0 update by G. Chilton  703-848-0951",
						 szAppName, MB_OK | MB_ICONEXCLAMATION);
				break;
			default:
				return DefWindowProc(hWnd, iMessage, wParam, lParam);
		};
		break;

	case    WM_SYSCOLORCHANGE:
		DeleteObject(hBarBrush);
		DeleteObject(hBackgroundBrush);
		hBarBrush = CreateSolidBrush(GetSysColor(COLOR_APPWORKSPACE));
		hBackgroundBrush = CreateSolidBrush(GetSysColor(COLOR_BACKGROUND));
		SetClassWord(hWnd,GCW_HBRBACKGROUND,hBackgroundBrush);
		break;

	case    WM_TIMER:
		wFreeMem = (WORD) (GetFreeSpace (0) / 1024);
		if (wFreeMem > wTotalMem) wTotalMem = wFreeMem;
		if (wFreeMem != wPrevMem) InvalidateRect(hWnd, NULL, TRUE);
		wPrevMem = wFreeMem;
		break;		

	case	WM_PAINT:
		hDC = BeginPaint(hWnd, &ps);
		SelectObject(hDC, hBackgroundBrush);
		Rectangle(hDC, xCXBORDER, yCYBORDER, xCXICON + xCXBORDER, yCYICON + yCYBORDER);

                for (i=1; i <= 3; i++)
                  {
                  MoveTo(hDC, xCXBORDER, yCYBORDER + (yCYICON / 4 * i));
                  LineTo(hDC, xCXBORDER + xCXICON, yCYBORDER + (yCYICON / 4 * i));
                  }
                
                for (i=1; i <= 7; i=i+2)
                  {
                  MoveTo(hDC, xCXBORDER + xCXICON / 8, yCYBORDER + (yCYICON / 8 * i));
                  LineTo(hDC, xCXBORDER + (xCXICON / 8 * 7), yCYBORDER + (yCYICON / 8 * i));
                  }
                
		SelectObject(hDC, hBarBrush);
		fMemRatio = (float) wFreeMem / (float) wTotalMem * (float) yCYICON;
		Rectangle(hDC, xCXICON / 4 + xCXBORDER,
					   yCYICON + yCYBORDER - (WORD) fMemRatio,
					   xCXICON / 4 * 3 + xCXBORDER ,
					   yCYICON + yCYBORDER);
		EndPaint(hWnd, &ps);
		break;

	case	WM_QUERYOPEN:
		break;

	case    WM_DESTROY  :   
		KillTimer(hWnd, 1);
		DeleteObject(hBarBrush);
		DeleteObject(hBackgroundBrush);
		PostQuitMessage(0);
		break;
	default:    
		return DefWindowProc(hWnd, iMessage, wParam, lParam);
        };
return 0L;
}
